#include "padlock_montmul.h"
#include "padlock.h"
#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <assert.h>
#include "global.h"
#include "rsaref.h"
#include "nn.h"

#define MAXLENGTH 260 
void padlock_montmul_assert_mmctx( montmul_context* mmctx)
{
	int i;

	assert( mmctx);

	for (i=0; i<8; ++i)
	{
		assert( mmctx->ptr[i]);
	}
	
	assert( mmctx->length>0);
}



void padlock_montmul_get_rm( montmul_context* mmctx)
{	
	uInt_32  span = MAXLENGTH;

	uInt_32* tmp = mmctx->ptr[TMP_PTR];	
	
	uInt_32 len = mmctx->length;

	uInt_32* rm = mmctx->ptr[RM_PTR];	
	
	memset ( tmp, 0, span * 4);

	memset ( rm, 0, span * 4);
	
	tmp [len] = 1;

	NN_Mult( rm, tmp, tmp, len + 1);

	NN_Mod( rm, rm, 2 * len + 1, mmctx->ptr[MOD_PTR], len);	

	memcpy ( mmctx->ptr[RM_PTR], rm, len * 4 );

}

montmul_context* padlock_montmul_begin()
{
	montmul_context* mmctx = (montmul_context*) malloc( sizeof( montmul_context));

	uInt_32* tmp = NULL;
	
	uInt_32 span = MAXLENGTH;

	int i;
	
	if (!mmctx)
	{
		return NULL;
	}
		
	mmctx->buffer = (uInt_32*) malloc(65535);
	
	if(NULL == mmctx->buffer)
	{
		free(mmctx);
		return NULL;
	}

	tmp = ALIGN16(mmctx->buffer);

	for (i = 0; i <8 ;++i )
	{
		mmctx->ptr[i] = tmp;
		mmctx->ptr[i]+= span * i;
	}
	
	mmctx->mzeroprime = 0;
	
	memset( mmctx->buffer, 0, 65535);
	
	return mmctx;
}

void padlock_montmul_close( montmul_context* mmctx)
{

	int i=0;

	mmctx->length = 0;

	mmctx->mzeroprime = 0;

	for (i=0; i<8; ++i)
	{
		mmctx->ptr[i] = NULL;
	}

	free ( mmctx->buffer);
	
	free ( mmctx);
}


void padlock_montmul_set_mod_len( montmul_context* mmctx, uInt_32* mod, uInt_32 len)
{
	memcpy( mmctx->ptr[MOD_PTR], mod, len * 4);

	mmctx->length = len;
	
	padlock_montmul_get_rm( mmctx);

}


void padlock_montmul_mm( uInt_32* A, uInt_32* B, uInt_32* dst, montmul_context* mmctx)
{
	uInt_32 len = mmctx->length;

	uInt_32 span = MAXLENGTH;

	uInt_32* a0 = mmctx->ptr[A_PTR];

	uInt_32* b0 = mmctx->ptr[B_PTR];

	uInt_32* dst0 = mmctx->ptr[DST_PTR];

	uInt_32* aPrime0 = mmctx->ptr[PRM_PTR];

	memset ( a0, 0, span * 4);

	memset ( b0, 0, span * 4);

	memset ( dst0, 0, span * 4);

	memset ( aPrime0, 0, span * 4);

	memcpy ( a0, A, span* 4);

	memcpy ( b0, B, span * 4);

	padlock_montmul_assert_mmctx( mmctx );

	assert( dst );
	
	/*
	mmctx->ptr[RM_PTR] == r^2
	MonPro(a0, r^2) 
	= a0 * r^2 * r^(-1) (mod n) 
	= a0 * r (mod n)
	*/
	padlock_pmm( a0, mmctx->ptr[RM_PTR], mmctx->ptr[MOD_PTR], aPrime0, len);
	
	padlock_pmm( aPrime0, b0, mmctx->ptr[MOD_PTR], dst0, len);

	memcpy ( dst, dst0, len * 4);
}



void padlock_mm(uInt_32* A, uInt_32* B, uInt_32* dst, uInt_32* mod, uInt_32 len)
{
	montmul_context* ctx = NULL;

//	if ( !padlock_pmm_available())
//	{
//		return ;
//	}
	 
	ctx = padlock_montmul_begin();
	
	padlock_montmul_set_mod_len( ctx, mod, len);

	padlock_montmul_mm( A, B, dst, ctx);

	padlock_montmul_close( ctx);

}
