/*
 * Copyright 1998-2005 VIA Technologies, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * VIA, AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "padlock.h"
#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <assert.h>
#include "via_ace.h"

extern void asm_pmm_op3(int temp, int dummy,void* mmCtx);

#define MAX_BITS 32768
#define MAX_WORDS (2 * MAX_BITS / 32) // set the value of MAX_WORDS be double of MAX_BITS
#define MAX_BYTES (MAX_WORDS * 4)	  // set the value of MAX_BYTES be double of MAX_BITS

typedef struct  
{
	uint_32  mZeroPrime;
	uint_32* A;
	uint_32* B;
	uint_32* T;
	uint_32* M;
	uint_32* scratch;
} mmCTX;

extern int asm_pmm_available();

int padlock_pmm_available()
{
	return asm_pmm_available();
}

static int bigIntGTE(uint_32 *A, uint_32 *B ,uint_32 LENGTH)
{
	int i;

	for (i = LENGTH + 1; i >= 0; i--) {
		if (A[i] > B[i]) return 1; //true
		if (A[i] < B[i]) return 0; //false
	}
	return 1;
}

static void bigIntSub(uint_32 *A, uint_32 *B, uint_32 LENGTH)
{
	uint_32 i,j;

	for (i = 0; i < LENGTH + 2; i++) {

		if (B[i] > A[i]) {
			j = i+1;
			A[j]--;
			while (A[j] == 0xFFFFFFFF)
				A[++j]--;
		}
		A[i] = A[i] - B[i];
	}
}

static uint_32 big_int_inv_mon( uint_32 *mod)/*����n0'*/
{
	uint_32 i;
	uint_64 x = 2, y = 1;
	uint_64 mZero = (uint_64)(mod[0]);

	for (i = 2; i <= 32; i++, x <<= 1) 
	{
		if (x < ((mZero * y) & ((x << 1) - 1))) 
		{
			y += x;
		}
	}
	return (uint_32)(x - y);
}
uint_32 bigInt_Digits(uint_32* data, int length)
{
  int i=length;
  while (--i>=0 && ! data[i]);
  return (i + 1);
}
void  bigInt_DigitMult (uint_32 result[2],uint_32 b,uint_32 c)
{

  uint_64 t = (uint_64)b*(uint_64)c;
  result[0]=(uint_32) (t&0xFFFFFFFF);
  result[1]=(uint_32) (t>>32);
}

/* Computes a = b + c*d, where c is a digit. Returns carry.
   Lengths: a[digits], b[digits], d[digits].
 */
 uint_32 bigInt_AddDigitMult (uint_32* a,uint_32* b,uint_32 c,uint_32* d,uint_32 digits)
{
  uint_32 carry=0,t[2];

  uint_32 i=digits;
  

  if (c == 0) return (0);

  while (i--)
  {
    bigInt_DigitMult (t, c, *d++);
    carry = ((*a = *b++ + carry) < carry);
    carry += ((*a++ += t[0]) < t[0]) + t[1];
  }
  
  return (carry);
}


//result = A*B
void big_int_mul(uint_32* A, uint_32* B, uint_32* result, uint_32 ndigits)
{
	uint_32* tmp = (uint_32*) malloc( (ndigits+1)*4+16);
	uint_32* t = (uint_32*) ALIGN16( tmp);
	uint_32 aDigits, bDigits, i;

	memset(tmp,0, (ndigits+1)*4+16);

	aDigits = bigInt_Digits(A, 1);
	bDigits = bigInt_Digits (B, ndigits);

  for (i = 0; i < aDigits; i++)
    t[i+bDigits] += bigInt_AddDigitMult (&t[i], &t[i], A[i], B, bDigits);
     
  memcpy( result, t , (ndigits+1)*4);
  
  free(tmp);

}

// a = b+c
uint_32 NN_Add(uint_32 *a, uint_32 *b, uint_32 *c, uint_32 digits)
{
  uint_32 ai, carry;
  unsigned int i;

  carry = 0;

  for (i = 0; i < digits; i++) {
    if ((ai = b[i] + carry) < carry)
      ai = c[i];
    else if ((ai += c[i]) < c[i])
      carry = 1;
    else
      carry = 0;
    a[i] = ai;
  }

  return (carry);
}

//to caculate montgomery multiplication by sw.
//dst = mon(A,B) = (A*B*R') mod M
void padlock_pmm_sw( uint_32* A, uint_32* B, uint_32* M, uint_32* dst, uint_32 ndigits)
{
	int nIndex, nInternal ;
	uint_32 carry = 0 ;
	uint_32 mZeroPrime = big_int_inv_mon(M);

	uint_32 *C_buf = (uint_32*)malloc((ndigits+2)*4 + 16) ;
	uint_32 *C     = (uint_32*)ALIGN16(C_buf) ;

	uint_32 *AiB0_buf = (uint_32*)malloc(2*4 + 16) ;
	uint_32 *AiB0     = (uint_32*)ALIGN16(AiB0_buf) ;

	uint_32 *qt_buf   = (uint_32*)malloc(2*4+16) ;
	uint_32 *qt       = (uint_32*)ALIGN16(qt_buf) ;
	uint_32 q ;

	uint_32 *AiB_buf  = (uint_32*)malloc((ndigits+1)*4+16) ;
	uint_32 *AiB      = (uint_32*)ALIGN16(AiB_buf) ;

	uint_32 *qM_buf   = (uint_32*)malloc((ndigits+1)*4+16) ;
	uint_32 *qM       = (uint_32*)ALIGN16(qM_buf) ;

	memset(C_buf, 0, (ndigits+1)*4 + 16) ;
	memset(AiB0_buf, 0, 2*4 + 16) ;
	memset(qt_buf, 0, 2*4 + 16) ;
	memset(AiB_buf, 0, (ndigits+1)*4 + 16) ;
	memset(qM_buf,  0, (ndigits+1)*4 + 16) ;

	for (nIndex=0; nIndex<ndigits; nIndex++)
	{
		big_int_mul(&A[nIndex], &B[0], AiB0, 1) ;
		carry = NN_Add(qt, &C[0], &AiB0[0], 1) ;
		qt[1] = 0 ;
		big_int_mul(&mZeroPrime, qt, qt, 1) ;
		q = qt[0] ;

		big_int_mul(&A[nIndex], B, AiB, ndigits) ;
		big_int_mul(&q, M, qM, ndigits) ;

		carry = NN_Add(C, C, AiB, ndigits+1) ;
		C[ndigits+1] += carry ;
		carry = NN_Add(C, C, qM,  ndigits+1) ;
		C[ndigits+1] += carry ;
		for (nInternal=1; nInternal<=(ndigits+2); nInternal++)
		{
			C[nInternal-1] = C[nInternal] ;
		}
		C[nInternal] = 0 ;
	}

	if (bigIntGTE(C, M, ndigits))
		bigIntSub(C, M, ndigits);

	memcpy(dst, C, ndigits*4);
 
	free(C_buf) ;
	free(AiB0_buf) ;
	free(qt_buf) ;
	free(qM_buf) ;
	free(AiB_buf) ;
	return ;

}
void padlock_pmm( uint_32* A, uint_32* B, uint_32* M, uint_32* dst, uint_32 ndigits)
{
	if(padlock_pmm_available())
	{
		mmCTX ctx ;
		uint_32 i = 0 ;
		unsigned char *buf = (unsigned char*)malloc(4 * 8 + 15);
		int dummy = 0;
		int	nbits = ndigits << 5;

		uint_32 *a_temp_buf = (uint_32*)malloc(MAX_BYTES + 16);
		uint_32 *b_temp_buf = (uint_32*)malloc(MAX_BYTES + 16);
		uint_32 *t_temp_buf = (uint_32*)malloc(MAX_BYTES + 16);
		uint_32 *m_temp_buf = (uint_32*)malloc(MAX_BYTES + 16);
		uint_32 *scratch = NULL;

		/* PMM hardware require:
		1)the A, B, M, T in the mmCTX should be pointer to a buffer with MAX_BYTES bytes
		2)the buffer should be 16-byte aligned.
		*/
		uint_32 *abuf = NULL;
		uint_32 *bbuf = NULL;
		uint_32 *mbuf = NULL;
		uint_32 *tbuf = NULL;

		if(NULL == buf || NULL == a_temp_buf || NULL == b_temp_buf || NULL == t_temp_buf || NULL == m_temp_buf)
		{
			free(buf);
			free(a_temp_buf);
			free(b_temp_buf);
			free(t_temp_buf);
			free(m_temp_buf);
			return ;
		}
		
		scratch= (uint_32*)ALIGN16(buf);

		/* PMM hardware require:
		1)the A, B, M, T in the mmCTX should be pointer to a buffer with MAX_BYTES bytes
		2)the buffer should be 16-byte aligned.
		*/
		abuf = (uint_32*)ALIGN16(a_temp_buf);
		bbuf = (uint_32*)ALIGN16(b_temp_buf);
		mbuf = (uint_32*)ALIGN16(m_temp_buf);
		tbuf = (uint_32*)ALIGN16(t_temp_buf);

		memset((unsigned char*)a_temp_buf, 0, (MAX_BYTES + 16));
		memset((unsigned char*)b_temp_buf, 0, (MAX_BYTES + 16));
		memset((unsigned char*)m_temp_buf, 0, (MAX_BYTES + 16));
		memset((unsigned char*)t_temp_buf, 0, (MAX_BYTES + 16));
		memset(buf, 0, 4 * 8 + 15);

		/* Copy the value to the 16-byte aligned buffer */
		memcpy((unsigned char*)abuf, (unsigned char*)A, (ndigits * 4));
		memcpy((unsigned char*)bbuf, (unsigned char*)B, (ndigits * 4));
		memcpy((unsigned char*)mbuf, (unsigned char*)M, (ndigits * 4));

		ctx.mZeroPrime = big_int_inv_mon(M);
		ctx.A = abuf;
		ctx.B = bbuf;
		ctx.T = tbuf;
		ctx.M = mbuf;
		ctx.scratch = scratch;        

		//MONTMUL(nbits, dummy,  &ctx);
		asm_pmm_op3(nbits, dummy,  &ctx);

		if (bigIntGTE(tbuf, mbuf, ndigits))
			bigIntSub(tbuf,	mbuf, ndigits);

		/* Copy the value to the dst buffer */
		memcpy((unsigned char*)dst, (unsigned char*)tbuf, (ndigits * 4));

		free(buf);
		free(a_temp_buf);
		free(b_temp_buf);
		free(m_temp_buf);
		free(t_temp_buf);
	}
	else//not available, then SW.
	{
		padlock_pmm_sw(A, B, M, dst, ndigits);
	}
}
