/*
 * Copyright 1998-2005 VIA Technologies, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * VIA, AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


#include "padlock.h"
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include "via_sha.h"
#include "via_ace.h"

                
SHA_RESULT padlock_phe_sha256(unsigned char *src, int nbytes,unsigned char* dst)
{
	
	int j=0;	
	unsigned int* temp_buffer = (unsigned int *)malloc(128+16);
	unsigned int x[8];
	/* The total memory allocated at ES:EDI (or ES:DI) must be 128 bytes. 
	REP XSHA microcode requires a 16-byte aligned bu.er to load the PHE 
	hardware. Only the hash pointer at EDI is guaranteed to be aligned.*/
	unsigned int* hash;
	int tmplen = nbytes;
	
	if(temp_buffer == NULL)
	{
		return SHA_FAILED;
	}
	hash=ALIGN16(temp_buffer);
	
	hash[0] = 0x6a09e667;
	hash[1] = 0xbb67ae85;
	hash[2] = 0x3c6ef372;
	hash[3] = 0xa54ff53a;
	hash[4] = 0x510e527f;
	hash[5] = 0x9b05688c;
	hash[6] = 0x1f83d9ab;
	hash[7] = 0x5be0cd19;

	tmplen = asm_sha256_op3(src,hash,nbytes);
	if (nbytes != tmplen)
	{
		printf("PHE error bytes returned by SHA256 instrunction!");
		free (temp_buffer);
		return SHA_FAILED;
	}

	/*Thus the calling program will need to  BSWAP each 32-bit DWORD 
	of the hash to produce the correct result if it is to be communicated
	to any other program, process, or user.*/
	for (j=0;j<8;++j)
	{
	   x[j]=bswap_32(hash[j]);
	}
	memcpy(dst,&x,32);
	free(temp_buffer);
	return SHA_SUCCEEDED; 

}

SHA_RESULT padlock_phe_sha1(unsigned char *src, int nbytes,unsigned char* dst)
{

	int j=0;
	unsigned int* temp_buffer = (unsigned int *)malloc(128+16);
	unsigned int x[5];
	/* The total memory allocated at ES:EDI (or ES:DI) must be 128 bytes. 
	REP XSHA microcode requires a 16-byte aligned bu.er to load the PHE 
	hardware. Only the hash pointer at EDI is guaranteed to be aligned.*/
	unsigned int*  hash = NULL;
	int tmplen = nbytes;
	
	if(temp_buffer == NULL)
	{
		return SHA_FAILED;
	}
	hash=ALIGN16(temp_buffer);

	hash[0] = 0x67452301;
	hash[1] = 0xefcdab89;
	hash[2] = 0x98badcfe;
	hash[3] = 0x10325476;
	hash[4] = 0xc3d2e1f0;

	tmplen = asm_sha1_op3(src,hash,nbytes);
	if (nbytes != tmplen)
	{
		printf("PHE error bytes returned by SHA1 instrunction!");
		free (temp_buffer);
		return SHA_FAILED;
	}
/*Thus the calling program will need to  BSWAP each 32-bit DWORD 
	of the hash to produce the correct result if it is to be communicated
	to any other program, process, or user.*/

	for (j=0;j<5;++j)
	{
	   x[j]=bswap_32(hash[j]);
	}
	memcpy(dst,&x,20);
	free(temp_buffer);	
	return SHA_SUCCEEDED; 
}

int padlock_phe_available()
{
	return asm_phe_available();
}


int padlock_phe_partial_available()
{
	return asm_is_nano();
}

struct phe_sha_context* padlock_phe_partial_sha1_init()
{
	struct phe_sha_context * ctx = (struct phe_sha_context *)malloc(sizeof(struct phe_sha_context));

	ctx->length = 0;

	ctx->hash[0] = 0x67452301;
	ctx->hash[1] = 0xefcdab89;
	ctx->hash[2] = 0x98badcfe;
	ctx->hash[3] = 0x10325476;
	ctx->hash[4] = 0xc3d2e1f0;

	return ctx;
}

SHA_RESULT padlock_phe_partial_sha1_update(struct phe_sha_context* ctx, unsigned char *src, int nbytes)
{
	int tmplen = 0;
	//1.test nbytes is multiple of 64
	if(nbytes%64)
	{
		return SHA_FAILED;
	}

	//2.call cpu instruction
	asm_partial_sha1_op3(src,ctx->hash,nbytes/64);

	//4.arrive here, then succeed, accumulate length
	ctx->length += nbytes;

	return SHA_SUCCEEDED;
}

SHA_RESULT padlock_phe_partial_sha1_final(struct phe_sha_context* ctx, unsigned char *src, int nbytes, unsigned char *dst)
{
	unsigned char padding[128] = {0};
	unsigned int swap[5] = {0};
	int len = 0;		//last block of data's length
	int temp = 0;		//for bswap length
	//1.get total length of data
	ctx->length += nbytes;

	//2.padding
	// | data    | 1000...  | len    |
	// | d bits  | k zeros  | 64bits |
	// d+k+1 = 448 mod 512
	len = nbytes%64;
	if(len)
		memcpy(padding,src+nbytes-len,len);

	padding[len] = 0x80;
	if(len<56)
	{
		*((uint_64*)(padding+56)) = ctx->length * 8;
		temp = *((int*)(padding+56));
		*((uint_32*)(padding+56)) = bswap_32(*((uint_32*)(padding+60)));
		*((uint_32*)(padding+60)) = bswap_32(temp);
	}
	else
	{
		*((uint_64*)(padding+120)) = ctx->length * 8;
		temp = *((int*)(padding+120));
		*((uint_32*)(padding+120)) = bswap_32(*((uint_32*)(padding+124)));
		*((uint_32*)(padding+124)) = bswap_32(temp);
	}

	//3.call cpu instruction
	if(nbytes!=len)
	{
		asm_partial_sha1_op3(src,ctx->hash,(nbytes-len)/64);
	}

	asm_partial_sha1_op3(padding,ctx->hash,len<56?1:2);

	//5.copy result from ctx->hash to dst
	for (temp=0;temp<5;++temp)
	{
	   swap[temp]=bswap_32(ctx->hash[temp]);
	}

	memcpy(dst, swap, 20);

	//6.arrive here, then succeed
	free(ctx);
	return SHA_SUCCEEDED;
}

struct phe_sha_context* padlock_phe_partial_sha256_init()
{
	struct phe_sha_context * ctx = (struct phe_sha_context *)malloc(sizeof(struct phe_sha_context));

	ctx->length = 0;

	ctx->hash[0] = 0x6a09e667;
	ctx->hash[1] = 0xbb67ae85;
	ctx->hash[2] = 0x3c6ef372;
	ctx->hash[3] = 0xa54ff53a;
	ctx->hash[4] = 0x510e527f;
	ctx->hash[5] = 0x9b05688c;
	ctx->hash[6] = 0x1f83d9ab;
	ctx->hash[7] = 0x5be0cd19;
	return ctx;
}

SHA_RESULT padlock_phe_partial_sha256_update(struct phe_sha_context* ctx, unsigned char *src, int nbytes)
{
	int tmplen = 0;
	//1.test nbytes is multiple of 64
	if(nbytes%64)
	{
		return SHA_FAILED;
	}

	//2.call cpu instruction
	asm_partial_sha256_op3(src,ctx->hash,nbytes/64);

	//4.arrive here, then succeed, accumulate length
	ctx->length += nbytes;

	return SHA_SUCCEEDED;
}

SHA_RESULT padlock_phe_partial_sha256_final(struct phe_sha_context* ctx, unsigned char *src, int nbytes, unsigned char *dst)
{
	unsigned char padding[128] = {0};
	unsigned int swap[8] = {0};
	int len = 0;		//last block of data's length
	int temp = 0;		//for bswap length
	//1.get total length of data
	ctx->length += nbytes;

	//2.padding
	// | data    | 1000...  | len    |
	// | d bits  | k zeros  | 64bits |
	// d+k+1 = 448 mod 512
	len = nbytes%64;
	if(len)
		memcpy(padding,src+nbytes-len,len);

	padding[len] = 0x80;
	if(len<56)
	{
		*((uint_64*)(padding+56)) = ctx->length * 8;
		temp = *((int*)(padding+56));
		*((uint_32*)(padding+56)) = bswap_32(*((uint_32*)(padding+60)));
		*((uint_32*)(padding+60)) = bswap_32(temp);
	}
	else
	{
		*((uint_64*)(padding+120)) = ctx->length * 8;
		temp = *((int*)(padding+120));
		*((uint_32*)(padding+120)) = bswap_32(*((uint_32*)(padding+124)));
		*((uint_32*)(padding+124)) = bswap_32(temp);
	}

	//3.call cpu instruction
	if(nbytes!=len)
	{
		asm_partial_sha256_op3(src,ctx->hash,(nbytes-len)/64);
	}

	asm_partial_sha256_op3(padding,ctx->hash,len<56?1:2);

	//5.copy result from ctx->hash to dst
	for (temp=0;temp<8;++temp)
	{
	   swap[temp]=bswap_32(ctx->hash[temp]);
	}

	memcpy(dst, swap, 32);

	//6.arrive here, then succeed
	free(ctx);
	return SHA_SUCCEEDED;
}